;; ir_comm - Copyright 2018 by Michael Kohn
;; Email: mike@mikekohn.net
;;   Web: http://www.mikekohn.net/
;;
;; Read data from a 38kHz IR sensor for Syma S107 remote.
;;

; Current interrupt is every 0.05ms

; start: 2ms on / 2ms off        40 / 40
;
;     0: 0.3ms on/ 0.3ms off     6 / 6 
;     1: 0.3ms on/ 0.6ms off     6 / 12
;                                
;  stop: 0.3 on/ 0.3 off?

.define IR_ABORT ir_comm_read

IR_PIN equ 1
HEADER equ 40
SHORT equ 6
LONG equ 12
;IR_DATA equ SRAM_START
;YAW equ IR_DATA + 0
;PITCH equ IR_DATA + 1
;THROTTLE equ IR_DATA + 2
;YAW_CORRECTION equ IR_DATA + 3

; ir_comm_read()
ir_comm_read:
  sbic PINB, IR_PIN   ; if (IR) break;
  rjmp ir_comm_read

  ;; Point Z to SRAM, so the IR remote data can be put there
  ldi r30, IR_DATA & 0xff
  ldi r31, IR_DATA >> 8

  ;; Check for the IR header signal (2ms on / 2ms off)
  ldi r20, 0
ir_wait_start_bit:
  cpi r20, HEADER + 2
  brsh IR_ABORT     ; if (r20 longer header time) abort;
  sbis PINB, IR_PIN ; if (no IR) break;
  rjmp ir_wait_start_bit

  cpi r20, HEADER - 2
  brlo IR_ABORT     ; if (r20 wasn't long enough) abort;

  ldi r20, 0
ir_wait_start_bit_off:
  cpi r20, HEADER + 2
  brsh IR_ABORT     ; if (r20 longer header time) abort;
  sbic PINB, IR_PIN ; if (IR) break;
  rjmp ir_wait_start_bit_off

  ;; Now read in 32 bits of IR data into SRAM to SRAM + 3

  ldi r22, 4     ; byte_count = 4

ir_read_byte:
  ldi r21, 0     ; current_byte = 0
  ldi r23, 8     ; bit_count = 8

ir_read_bit:
  ldi r20, 0
ir_read_bit_on:
  cpi r20, SHORT + (SHORT / 2)
  brsh IR_ABORT     ; if (r20 longer than short time) abort;
  sbis PINB, IR_PIN ; if (no IR) break;
  rjmp ir_read_bit_on

  ldi r20, 0
ir_read_bit_off:
  cpi r20, LONG + (SHORT / 2)
  brsh IR_ABORT     ; if (r20 longer than long time) abort;
  sbic PINB, IR_PIN ; if (IR) break;
  rjmp ir_read_bit_off

  lsl r21        ; current_byte <<= 1;
  cpi r20, SHORT + (SHORT / 2)
  brlo ir_not_a_one
  sbr r21, 1     ; if (r20 longer than a short) current_byte |= 1;
ir_not_a_one:

  dec r23        ; bit_count--;
  brne ir_read_bit  ; do while(bit_count != 0)

  st Z+, r21     ; SRAM[Z++] = curent_byte
  dec r22        ; byte_count--;
  brne ir_read_byte

  ;; wait for IR to stop
  ldi r20, 0
read_last_bit_on:
  cpi r20, SHORT + (SHORT / 2)
  brsh IR_ABORT     ; if (r20 longer than long time) abort;
  sbis PINB, IR_PIN ; if (IR) break;
  rjmp read_last_bit_on

  ret

