;; uart_sw - Copyright 2018 by Michael Kohn
;; Email: mike@mikekohn.net
;;   Web: http://www.mikekohn.net/
;;
;; Software UART routines
;;

RX_PIN equ 2
TX_PIN equ 3
INTERRUPTS_PER_BIT equ 10

; 20,000,000 / 9600 = 2083.3 cycles per bit
; 2083.3 / 208 = 10 interrupts per bit

; void uart_sw_send(r24)  : r14, r17 trashed
uart_sw_send:
  ;sbi PORTD, 4

  ;; Set timer to fit 9600 baud better
  ldi r17, (208 >> 8)
  sts OCR1AH, r17
  ldi r17, (208 & 0xff)
  sts OCR1AL, r17

  ;; Start bit is 0
  sts TCNT1H, r0
  sts TCNT1L, r0
  clr r20
  cbi PORTD, TX_PIN
wait_start_bit:
  cpi r20, INTERRUPTS_PER_BIT
  brne wait_start_bit

  ;; Shift out 8 bit LSb first + start bit
  ldi r17, 9
shift_next_bit:
  sts TCNT1H, r0
  sts TCNT1L, r0
  sbrc r24, 0
  sbi PORTD, TX_PIN
  sbrs r24, 0
  cbi PORTD, TX_PIN
  sec
  ror r24
  clr r20
wait_data_bit:
  cpi r20, INTERRUPTS_PER_BIT
  brne wait_data_bit
  dec r17
  brne shift_next_bit

  ;; Put timer back to normal
  ldi r17, (INTERRUPT_COUNT >> 8)
  sts OCR1AH, r17
  ldi r17, (INTERRUPT_COUNT & 0xff) ; compare to 1000 clocks (0.05ms)
  sts OCR1AL, r17

  ;cbi PORTD, 4

  ret

uart_sw_send_string:
  lpm r24, Z+
  cpi r24, 0
  breq uart_sw_send_string_done
  rcall uart_sw_send
  rjmp uart_sw_send_string
uart_sw_send_string_done:
  ret

uart_sw_init:
  sbi PORTD, TX_PIN
  ret

